
	list      p=12F675        		; list directive to define processor
	#include <p12f675.inc>          ; processor specific variable definitions
	errorlevel  -302               ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;****************************************************************************
;Defines
;****************************************************************************
#define	BANK1		banksel 0x80	;Select Bank1
#define	BANK0		banksel 0x00	;Select Bank0
#define count		0x20			; used to adjust the delay time
#define MaxRead_H	0x21
#define MaxRead_L	0x22
#define MinRead_H	0x23
#define MinRead_L	0x24
#define FirstRead_H	0x25
#define FirstRead_L	0x26
#define CurrentRead_H	0x27
#define	CurrentRead_L	0x28
#define temp		0x29
#define SourceL		0x2A		;substraction varable low byte
#define SourceH		0x2B		;substraction varable high byte
#define DestL		0x2C		;substraction varable low byte
#define DestH		0x2D		;substraction varable high byte
#define display		0x2F		; number to be transmitted
#define delay_cntr	0x30		; counter for serial delay routine
#define Display		0x34		; byte to be transmitted
#define NumH		0x35		; varible to be converted to BCD,High byte
#define NumL		0x36		; varible to be converted to BCD,low byte	
#define TenK		0x37		; BCD ten thousands
#define Thou		0x38		; BCD thousands
#define Hund		0x39		; BCD hundreds
#define Tens		0x3A		; BCD tens
#define Ones		0x3B		; BCD ones
#define Alt_H		0x3C
#define Alt_L		0x3D

;****************************************************************************
;Reset Vector
;****************************************************************************
	ORG     0x000             ; processor reset vector
	nop						  ; Inserted For ICD2 Use
	goto    Init              ; go to beginning of program
;****************************************************************************
;Initialization
;****************************************************************************
Init
	BANK1				; Switch to Bank1
	call    0x3FF      	; retrieve factory calibration value
	movwf   OSCCAL      ; update register with factory cal value 
	movlw	B'11001101'	; pin 6,3 & 2 are outputs and pins 1,4,5,6,7 are inputs_
	movwf	TRISIO		;_bit0=7,bit1=6,bit2=5,bit3=4,bit4=3,bit5=2,
						;_1=input, 0=output
	movlw	B'11010111' ; set internal clock(5),prescale =1:256 assigned to TIMER0_
	movwf	OPTION_REG	;_bit0-2=prescale(100=1:32),bit3=prescale=TIMER0 (=0)_
						;_bit4=trailing edge,bit5=internal clock(0)
						;_bit6=int (1=rising), bit7=pullups (diabled=1)
	clrf	VRCON		; Vref Off (power off the comparator voltage,saves power)
	movlw	b'01010001'	; pin 7 as anlog, Tad=16us
	movwf	ANSEL		;_bit0=7,bit1=6,bit2=5,bit3=3 (1=analog,0=digital)
						;_bit4-6= A/D clock osc value (101=16)
	BANK0				; BANK 0
	clrf	TMR0		; clear TIMER1
	clrf	GPIO		; Clear Port
	movlw	b'00000111'		
	movwf	CMCON		; Comparator Off, bit0,1,2=1 (00000111) 
	bsf		ADCON0,0	; turn A to D converter to "on", bit0=1
	bsf		ADCON0,7	; right justified A/D, bit 7=1
	movlw	b'00101111'	; used to pause for 8 TMR0 overflows_
	movwf	count		; to give everthing time to warmup_
	call	ShortDelay	; also need at least 4 micro seconds before starting A to D_
	bsf		ADCON0,1	; Start A/D conversion of voltage on pin 7
Start
	btfss	PIR1,6		; PIR1 bit6 =1 when AD conversion is complete
	goto	Start		; start taking and recording pressure changes
	bcf		PIR1,6
	nop
	call 	Baseline	; record the first pressure reading
	nop
	movf	MinRead_H,w
	movwf	NumH		; high order byte to be converted to BCD
	movf	MinRead_L,w
	movwf	NumL		; Low order byte to be converted to BCD
	call	BCD
	call	BlinkLED
	nop
	nop
	nop
	nop
	bsf		ADCON0,1	; Start A/D conversion of voltage on pin 7
;****************************************************************************
;MAIN - Main Routine
;****************************************************************************
Main
	btfss	PIR1,6		; PIR1 bit6 =1 when AD conversion is complete
	goto	Main
	bcf		PIR1,6		;
	call	Update		; compare the ad result and update max and min values
	call	Low_
	nop
	nop
	nop
	nop	
	btfss	GPIO,3		; switch on pin4 is closed (grounded)
	call	transmitt	; if switch on pin 4 is closed display the minimum value
	nop
	nop
	nop
	nop
	bsf		ADCON0,1	; Start A/D conversion of voltage on pin 7
	goto	Main	
;********************************subroutines ***********************
;*******************************************************************
Baseline				;establish the initial pressure readings
	movf	ADRESH,0	;move the AD result to w register
	movwf	FirstRead_H	;move to the First reading register
	movwf	MinRead_H
	movwf	MaxRead_H
	movwf	CurrentRead_H
	BANK1
	movf	ADRESL,0
	BANK0
	nop
	movwf	FirstRead_L
	movwf	MinRead_L
	movwf	MaxRead_L
	movwf	CurrentRead_L
	return
;*****************************************************************	
Update					; sub routine which updates the maximum 
						; and minimum registers 
					
	BANK1
	movf	ADRESL,0
	BANK0
	movwf	CurrentRead_L
	movwf	SourceL
	nop
	BANK0
	nop
	movf	ADRESH,0		; move the AD presure result to _
	movwf	CurrentRead_H	; the CurrentRead register.
	
	movwf	SourceH
	movf	MaxRead_H,0		; move the maxium presure reading to _
	movwf	DestH
	movf	MaxRead_L,0
	movwf	DestL
	call	Subtract	
	btfss	STATUS,0
	call	MaxUpdate

	movf	CurrentRead_H,0
	movwf	SourceH
	movf	CurrentRead_L,0
	movwf	SourceL
	movf	MinRead_H,0		; move the maxium presure reading to _
	movwf	DestH
	movf	MinRead_L,0
	movwf	DestL
	call	Subtract	
	btfsc	STATUS,0
	call	MinUpdate
	return

MaxUpdate
	movf	CurrentRead_L,0	; the CurrentReading lower bits register.
	movwf	MaxRead_L	; move the maxium presure reading lower bits to _
	movf	CurrentRead_H,0	; move current presure reading lower bits back into W register.
	movwf	MaxRead_H	;skip this step else replace maxRead with CurrentRead
	return

MinUpdate
	movf	CurrentRead_L,0	; the CurrentReading lower bits register.
	movwf	MinRead_L	; move the maxium presure reading lower bits to _
	movf	CurrentRead_H,0	; move current presure reading lower bits back into W register.
	movwf	MinRead_H	;skip this step else replace maxRead with CurrentRead
	return
	
;*****************************************************************
Subtract	; a 16 bit subtraction sub program.  Source - Dest.  
			; will return a STATUS,0=1 if a negative result is produced.
			; the high order byte is in "H" the low order byte is in "L"
	movf	SourceL,W	
	subwf	DestL,1		
	movf	SourceH,W	
	btfss	STATUS,C
	incfsz	SourceH,W	
	subwf	DestH,1		
	return
;******************************************************************		
High_					; this routine turns on LED1 for one delay count
	bcf		INTCON,2	; clear the interupt bit from the timer overflow
	bsf		GPIO,4		; turn on LED on pin 3
	clrf	TMR0		; reset timer 1
	movlw	0x04		; used to turn  LED on for 4 TMR0 overflows
	movwf	count
	call	ShortDelay	; delay 256*256 clock cycles
	bcf		GPIO,4		; turn LED on pin 3 off
	movlw	0x08
	movwf	count
	call	ShortDelay	; delay for 256*256 clock cycles
	return
;*************************************************************************	
Low_					; this routine turns on LED2 for 1 delay counts
	bcf		INTCON,2	; clear the interupt bit from the timer overflow
	bsf		GPIO,1		; turn on LED on pin 3
	clrf	TMR0		; reset timer 1
	movlw	0x04		; used to turn  LED on for F TMR0 overflows
	movwf	count
	call	ShortDelay	; delay 256*256 clock cycles
	bcf		GPIO,1		; turn LED on pin 3 off
	movlw	0x08
	movwf	count
	call	ShortDelay	; delay 256*256 clock cycles
	return
;***************************************************************	
ShortDelay				;pauses until TMR0 over flows
	btfss	INTCON,2	; timer 0 overflow interup flag
	goto	ShortDelay
	bcf		INTCON,2
	decfsz	count,1
	goto	ShortDelay
	bcf		INTCON,2
	return
;***************************************************************
BCD						; convert number in NumH and NumL converts
						; to 5 digit BCD number contained in 
						; registers TenK, Thou, Hund, Tens, Ones
	swapf	NumH,w
	andlw	b'00001111'
	addlw	b'11110000'
	movwf	Thou
	addwf	Thou,f
	addlw	b'11100010'
	movwf	Hund
	addlw	b'00110010'
	movwf	Ones
	
	movf	NumH,w
	andlw	b'00001111'
	addwf	Hund,f
	addwf	Hund,f
	addwf	Ones,f
	addlw	b'11101001'
	movwf	Tens
	addwf	Tens,f
	addwf	Tens,f
	
	swapf	NumL,w
	andlw	b'00001111'
	addwf	Tens,f
	addwf	Ones,f
	
	rlf		Tens,f
	rlf		Ones,f
	comf	Ones,f
	rlf		Ones,f
	
	movf	NumL,w
	andlw	b'00001111'
	addwf	Ones,f
	rlf		Thou,f
	
	movlw	b'00000111'
	movwf	TenK
	movlw	b'00001010'
Lb1
	addwf	Ones,f
	decf	Tens,f
	btfss	STATUS,0
	goto	Lb1
Lb2
	addwf	Tens,f
	decf	Hund,f
	btfss	STATUS,0
	goto	Lb2
Lb3
	addwf	Hund,f
	decf	Thou,f
	btfss	STATUS,0
	goto	Lb3
Lb4
	addwf	Thou,f
	decf	TenK,f
	btfss	STATUS,0
	goto	Lb4
	
	return
;***************************************************************
BlinkLED					; Blinks the LEDs to count out the
						; contents of Hund Tens and Ones registers
	bsf		GPIO,1
	movlw	0x20		; used to turn  LED on for 512 TMR0 overflows
	movwf	count
	call	ShortDelay
	bcf		GPIO,1
	incf	Hund,f
	decfsz	Hund,f
	call	blinkHun
	call 	Low_
	incf	Tens,f
	decfsz	Tens,f
	call	blinkTen
	call	Low_
	incf	Ones,f
	decfsz	Ones,f
	call	blinkOne
	call	Low_
	movlw	0x10		; used to turn  LED on for 256 TMR0 overflows
	movwf	count
	call	ShortDelay
	return
	
blinkHun
	call	High_
	decfsz	Hund,f
	goto	blinkHun
	return
blinkTen
	call	High_
	decfsz	Tens,f
	goto	blinkTen
	return	
blinkOne
	call	High_
	decfsz	Ones,f
	goto	blinkOne
	return
	
;***************************************************************
transmitt				; blink LEDs 
	movf	MinRead_H,w
	movwf	NumH		; high order byte to be converted to BCD
	movf	MinRead_L,w
	movwf	NumL		; Low order byte to be converted to BCD
	call	BCD
	call	BlinkLED
	btfsc	GPIO,3		; switch on pin4 is closed
	goto	transmitt

	movf	CurrentRead_H,0
	movwf	Display		; binary number to be transmitted

	movf	CurrentRead_L,0
	movwf	Display
	movf	FirstRead_H,0
	movwf	Display

	movf	FirstRead_L,0
	movwf	Display

	movf	MaxRead_H,0
	movwf	Display

	movf	MaxRead_L,0
	movwf	Display
	movf	MinRead_H,0
	movwf	Display

	movf	MinRead_L,0
	movwf	Display

	goto	transmitt
	
	END

